<?php

    /**
     * Exit if accessed directly
     */
    if (!defined('ABSPATH')) exit;

    define('PLUGIN_VERSION','1.2.0');


    // カート追加時custom_add_to_cart()呼び出し
    add_action('woocommerce_add_to_cart', array('WC_Alij_Payment', 'custom_add_to_cart'));

    /**
     * Another Lane Payment Gateway class
     */
    class WC_Alij_Payment extends WC_Payment_Gateway
    {

        protected $msg = array();

        public function __construct()
        {

            $this->id = 'wc_alij_payment';
            $this->method_title = 'アナザーレーン';
            $this->method_description = 'アナザーレーン決済ページを経由し決済を行います。';
            $this->icon = WP_PLUGIN_URL . "/" . plugin_basename(dirname(__FILE__)) . '/images/logo.gif';
            $this->has_fields = false;
            //$this->order_button_text    = __( '決済手続ページに進む', 'alij' );
            $this->init_form_fields();
            $this->init_settings();
            $this->title = $this->settings['title'];
            $this->description = $this->settings['description'];
            $this->site_id = $this->settings['SiteId'];
            $this->site_pass = $this->settings['SitePass'];
            $this->serverIP = $this->settings['serverIP'];
            $this->use_test = $this->settings['useTest'];
            $this->use_quick = $this->settings['useQuick'];
            $this->use_auto_complete = $this->settings['useAutoComplete'];
            $this->useAmountCheck = $this->settings['useAmountCheck'];
            $this->url = 'https://payment.alij.ne.jp/service/credit';
            $this->test_url = 'https://test-payment.alij.ne.jp/service/credit';

            // 国際化ファイルをロード
            //load_plugin_textdomain('alij', false, dirname( plugin_basename( __FILE__ ) ) . '/languages');

            add_action('init', array(&$this, 'check_alij_response'));
            //update for woocommerce >2.0
            // 'woocommerce_api_wc_alij_payment'と対応したURL(WC_API定義)にアクセス時、check_alij_response()呼び出し
            add_action('woocommerce_api_wc_alij_payment', array($this, 'check_alij_response'));


            if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
            } else {
                add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
            }

            add_action('woocommerce_receipt_wc_alij_payment', array(&$this, 'receipt_page'));

            if (function_exists('indatos_woo_alij_process_refund')) {
                $this->supports = array(
                    'products',
                    'refunds',
                );
            }
        }

        /**
         * Initialise Gateway Settings Form Fields
         */
        function init_form_fields()
        {

            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('有効/無効', 'alij'),
                    'type' => 'checkbox',
                    'label' => __('アナザーレーン決済を有効にする', 'alij'),
                    'default' => 'yes'),
                'title' => array(
                    'title' => __('決済方法', 'alij'),
                    'type' => 'text',
                    'description' => __('お支払時にユーザーに表示される決済方法の名称を設定します。', 'alij'),
                    'default' => __('アナザーレーン', 'alij')),
                'description' => array(
                    'title' => __('説明', 'alij'),
                    'type' => 'textarea',
                    'description' => __('お支払時にユーザーに表示される決済方法の説明を設定します。', 'alij'),
                    'default' => __('アナザーレーン決済ページを経由し決済を行います。', 'alij')),
                'SiteId' => array(
                    'title' => __('サイトID', 'alij'),
                    'type' => 'text'),
                'SitePass' => array(
                    'title' => __('サイトPASS', 'alij'),
                    'type' => 'text',
                    'description' => __('サイトID、サイトPASSは担当営業までお問い合わせください。')),
                'serverIP' => array(
                    'title' => __('サーバーIP', 'alij'),
                    'type' => 'text',
                    'description' => __('Callbackの送信元IPアドレスと設定値が一致しているかのチェックが行われます。チェックを行わない場合は、「0.0.0.0」で設定してください。')),
                'useTest' => array(
                    'title' => __('テスト環境', 'alij'),
                    'type' => 'checkbox',
                    'label' => __('テスト環境を使用する', 'alij'),
                    'default' => 'no',
                    'description' => __('テスト環境をご希望の際は担当営業までご連絡ください。')),
                'useQuick' => array(
                    'title' => __('クイックチャージ', 'alij'),
                    'type' => 'checkbox',
                    'label' => __('クイックチャージを使用する', 'alij'),
                    'default' => 'no',
                    'description' => __('クイックチャージをご希望の際は担当営業までご連絡ください。')),
                'useAutoComplete' => array(
                    'title' => __('受注管理オプション', 'alij'),
                    'type' => 'checkbox',
                    'label' => __('バーチャル商品の「処理中」をスキップする', 'alij'),
                    'default' => 'no',
                    'description' => __('バーチャル商品のみを含む受注のステータスを「完了」にマークします。')),
                'useAmountCheck' => array(
                    'title' => __('金額チェック', 'alij'),
                    'type' => 'checkbox',
                    'label' => __('金額チェックを使用する', 'alij'),
                    'default' => 'yes',
                    'description' => __('「有効」にすると、Woocommerceから送られてくる請求金額と、Callbackでの決済金額が一致しているかのチェックが行われます。')),
            );
        }

        /**
         *  There are no payment fields for AnotherLane, but want to show the description if set.
         **/
        function payment_fields()
        {
            if ($this->description)
                echo wpautop(wptexturize($this->description));
        }

        /**
         * Receipt Page
         * @param $order
         */
        function receipt_page($order)
        {
            $this->generate_alij_form($order);
        }

        /**
         * Process the payment and return the result
         * @param int $order_id
         * @return array
         */
        function process_payment($order_id)
        {
            $order = new WC_Order($order_id);
            return array(
                'result' => 'success',
                'redirect' => $order->get_checkout_payment_url(true),
            );
        }

        // カートに商品追加時、継続商品チェック
        public static function custom_add_to_cart()
        {
            $items = WC()->cart->get_cart();

            foreach ($items as $item => $values) {
                // product_idから商品属性の条件(slugs)を取得
                $product_id = $values['product_id'];
                $product_cont_terms = wc_get_product_terms($product_id, 'pa_alij_cont_enable', array('fields' => 'slugs'));
                $cont_mark = array_shift($product_cont_terms);

                // 継続商品(属性alij_cont_enableの条件slugsをtrueに設定した商品)が他の商品と同時にカートに存在する場合エラー
                if ($cont_mark == 'true' && count($items) > 1) {
                    // カートに入れる→商品追加の間のフックが見つからなかったため、商品追加後、最後の品物を削除
                    $temp = array_pop($items);
                    WC()->cart->remove_cart_item($temp['key']);
                    // エラーメッセージを表示
                    throw new Exception(sprintf(__('定期購入商品は個別に注文してください。', 'alij')));
                    break;
                }
            }
        }

        /**
         * Check for valid AnotherLane server callback to validate the transaction response.
         * or
         * Back to Thankyou page (or cart page) from AnotherLane payment site.
         **/
        function check_alij_response()
        {
            // kick処理の場合(GET)
            $params = $_GET;
            if (isset($params['SiteTransactionId'])) {
                $this->alij_kick($params);
            }
            
            // 戻り先遷移の場合(POST)
            $params = $_POST;
            if (isset($params['SiteTransactionId'])) {
                $this->alij_web($params);
            }

        }

        // kick処理の場合
        function alij_kick($params)
        {
            if (!isset($params['SiteTransactionId'])) {
                return;
            }
            
            try {
                $order = new WC_Order($params['SiteTransactionId']);
            } catch (Exception $e) {
                error_log(('[' . date_i18n('Y-m-d H:i:s') . '] alij.WARN: 該当オーダーが見つかりませんでした。'
                        . json_encode($params, JSON_UNESCAPED_UNICODE)) . '   ', 3, WP_CONTENT_DIR . '/uploads/wc-logs/alij_info.log');
                exit;
            }

            // 金額チェック
            if ($this->checkAmount($order, $params)) {
                error_log(('[' . date_i18n('Y-m-d H:i:s') . '] alij.WARN: オーダー金額と決済金額が一致していません。'
                        . json_encode($params, JSON_UNESCAPED_UNICODE)) . '   ', 3, WP_CONTENT_DIR . '/uploads/wc-logs/alij_info.log');
                exit;
            }

            // IPチェック
            $ip = $this->checkIP();
            if ($ip != '') {
                error_log(('[' . date_i18n('Y-m-d H:i:s') . '] alij.WARN: IPアドレスが確認できませんでした。' . $ip
                        . json_encode($params, JSON_UNESCAPED_UNICODE)) . '   ', 3, WP_CONTENT_DIR . '/uploads/wc-logs/alij_info.log');
                exit;
            }

            switch ($params['saleResult']) {
                case 'SALE':
                    $this->credit_completeOrder($params);
                    break;

                case 'PAYWAIT':
                    $this->processOrder($params);
                    break;

                // 戻り先URL{?wp-api=WC_Alij_Payment}を指定時、パラメータとして送られるため
                // cancel時ここにも入ってログ出力される
                default:
                    error_log(('[' . date_i18n('Y-m-d H:i:s') . '] alij.INFO: ' . json_encode($params, JSON_UNESCAPED_UNICODE))
                        . '   ', 3, WP_CONTENT_DIR . '/uploads/wc-logs/alij_info.log');
                    break;
            }
        }

        // 決済成功kick処理
        function credit_completeOrder($params)
        {
            if (!isset($params['SiteTransactionId'])) {
                return;
            }

            $order = new WC_Order($params['SiteTransactionId']);

            // 銀振入金超過の場合処理なし
            if (!($params['Result'] == 'MORE')) {
                WC()->cart->empty_cart();

                // 支払い待ちなら、受注管理画面にメモ出力
                if ($order->get_status() === 'pending') {
                    $order->add_order_note('アナザーレーン決済が完了しました。<br/>注文番号: '
                    . $_REQUEST['SiteTransactionId'] . '/取引ID: ' . $_REQUEST['TransactionId']
                    . '<br/>お支払方法:' . $params['paymentMethodName']);
                }

                // 受注管理オプション有効時
                if ($this->use_auto_complete == 'yes') {
                    // すべてバーチャル商品の場合決済ステータスを「完了」にする
                    $items = $order->get_items();
                    $virtual_order = null;
                    if (count($items) > 0) {
                        foreach ($items as $item) {
                            if ('line_item' == $item['type']) {
                                $product = $item->get_product();
                                if (!$product->is_virtual()) {
                                    $virtual_order = false;
                                    break;
                                } else {
                                    $virtual_order = true;
                                }
                            }
                        }
                    }
                    if ($virtual_order) {
                        $order->update_status('completed');
                    }
                }
                // 更新処理
                $order->payment_complete($_REQUEST['SiteTransactionId']);

            } else {
                error_log(('[' . date_i18n('Y-m-d H:i:s') . '] alij.INFO: ' . json_encode($params, JSON_UNESCAPED_UNICODE))
                    . '   ', 3, WP_CONTENT_DIR . '/uploads/wc-logs/alij_info.log');
            }
        }

        // 申し込み成功kick処理
        function processOrder($params)
        {
            if (!isset($params['SiteTransactionId'])) {
                return;
            }

            $order = new WC_Order($params['SiteTransactionId']);

            // 銀振入金不足の場合処理なし
            if (!($params['Result'] == 'LESS') && $order->get_status() != 'on-hold') {
                WC()->cart->empty_cart();
                // 受注管理画面にメモ出力
                $order->add_order_note('アナザーレーン決済申し込みが完了しました。<br/>注文番号: '
                    . $_REQUEST['SiteTransactionId'] . '/取引ID: ' . $_REQUEST['TransactionId']
                    . '<br/>お支払方法:' . $params['paymentMethodName']);
                // 更新処理(保留中)
                $order->update_status('on-hold');

            } else {
                error_log(('[' . date_i18n('Y-m-d H:i:s') . '] alij.INFO: ' . json_encode($params, JSON_UNESCAPED_UNICODE))
                    . '   ', 3, WP_CONTENT_DIR . '/uploads/wc-logs/alij_info.log');
            }
        }

        // 戻り先遷移の場合
        function alij_web($params)
        {
            
            if (!isset($params['SiteTransactionId'])) {
                return;
            }
            
            $order = new WC_Order($params['SiteTransactionId']);

            switch ($params['Result']) {
                // PAYWAITは申し込み成功時用
                case 'OK':
                case 'PAYWAIT':
                    $this->alij_success($order);
                    break;

                case 'NG':
                    $this->alij_cancel($order);
                    break;

                default:
                    error_log(('[' . date_i18n('Y-m-d H:i:s') . '] alij.INFO: ' . json_encode($params, JSON_UNESCAPED_UNICODE)) . '   ', 3, WP_CONTENT_DIR . '/uploads/wc-logs/alij_info.log');
                    break;
            }
        }

        // 決済成功時戻り先
        function alij_success($order)
        {
            $received_url = $order->get_checkout_order_received_url();
            wp_redirect($received_url);
            exit;
        }

        // 決済キャンセル時戻り先
        function alij_cancel($order)
        {
            $cancel_url = $order->get_cancel_order_url();
            wp_redirect($cancel_url);
            exit;
        }


        /**
         * Generate alij.net button link
         * @param $order_id
         * @return bool
         */
        public function generate_alij_form($order_id)
        {
            $order = new WC_Order($order_id);
            $tax = new WC_Tax();

            // 継続決済有効・無効の切り替え
            // product_idから商品属性の条件(slugs)を取得
            $items = $order->get_items();
            foreach ($items as $item) {

                $product_id = $item['product_id'];
                $product_cont_terms = wc_get_product_terms($product_id, 'pa_alij_cont_enable', array('fields' => 'slugs'));
                $product_amount_terms = wc_get_product_terms($product_id, 'pa_alij_cont_amount2', array('fields' => 'slugs'));
                $product_times_terms = wc_get_product_terms($product_id, 'pa_alij_cont_times', array('fields' => 'slugs'));

                $cont_mark = array_shift($product_cont_terms);
                $cont_amount_row = array_shift($product_amount_terms);
                $cont_times = array_shift($product_times_terms);

            }

            $cont_amount = null;
            $product_rates = 0;
            // 税率取得
            if (!empty($cont_amount_row)) {

                foreach ($items as $item) {
                    $temp_class = $item->get_tax_class();
                    $taxes = $tax->get_rates($temp_class);

                    foreach ($taxes as $tax) {
                        $product_rates += $tax['rate'];
                    }
                    $product_rates = round($product_rates / 100, 5);
                }

                // 送料取得
                $shipping_amount2 = $order->get_shipping_total() + $order->get_shipping_tax();

                // 個数取得
                $cont_quantities = $order->get_item_count();

                // 次回金額 = 「次回金額」に入力した数値 * 商品の税率 * 個数 + 送料
                $cont_amount = (($cont_amount_row * ($product_rates + 1)) * $cont_quantities) + ($shipping_amount2);
            }

            // 継続決済の場合 $amount2と$max_contを付与
            if ($cont_mark == 'true') {
                $use_cont = 1;
                $amount2 = $cont_amount;
                $max_cont = $cont_times;
                // その他決済
            } else {
                $amount2 = null;
                $max_cont = 1;
            }

            // ユーザー情報取得
            $user_info = get_userdata($order->get_user_id());
            // 送信パラメータ
            $alij_args = array(
                'SiteId' => $this->site_id,
                'SitePass' => $this->site_pass,
                'mail' => $user_info->user_email,
                'Amount' => $order->get_total(),
                'Amount2' => $amount2,
                'max_cont' => $max_cont,
                'SiteTransactionId' => $order_id,
                'WC_Version' => WC()->version,
                'WP_Version' => get_bloginfo('version'),
                'Plugin_Version' => PLUGIN_VERSION,
            );

            // プラグイン設定でクイックチャージ「有効」且つ会員の場合
            if ($this->use_quick == 'yes' && !is_null($user_info)) {
                $alij_args = $this->setQuickParameters($user_info, $alij_args, $use_cont);
            }

            // テスト環境
            if ($this->use_test == 'yes') {
                $processURI = $this->test_url;
            } else {
                $processURI = $this->url;
            }

            // クエリ文生成
            $destination_url = $processURI;
            $url = $destination_url . '?' . http_build_query($alij_args);
            // アナザー決済ページへリダイレクト
            return wp_redirect($url);
        }

        /**
         * 顧客ID/PASS設定
         * @param $user_info
         * @param $alij_args
         * @param $use_cont
         * @return array
         */
        function setQuickParameters($user_info, $alij_args, $use_cont)
        {
            // ユーザー登録必須項目がメールアドレスとパスのみの為email使用
            $quickParameter = array(
                'CustomerId' => $user_info->ID,
                'CustomerPass' => $user_info->user_email,
            );
            // 複数の継続を実行可能にするため、ユーザーIDとorder_idを合わせユニークなCustomerIdを発行する
            if ($use_cont === 1) {
                $quickParameter = array(
                    'CustomerId' => $user_info->ID . '_' . $alij_args['SiteTransactionId'],
                    'CustomerPass' => $user_info->user_email,
                );
            }
            return array_merge($alij_args, $quickParameter);
        }

        /**
         * 金額チェック
         * @param $order
         * @param $params
         * @return bool
         */
        function checkAmount($order, $params)
        {

            if ($this->useAmountCheck == 'no') {
                return false;
            }

            if (array_key_exists('billingAmount', $params)) {
                // オーダー合計金額と銀振などの請求金額一致ではない場合
                if ($order->get_data()['total'] != $params['billingAmount']) {
                    return true;
                }
            } else {
                // オーダー合計金額とクレジットカード決済などの請求金額一致ではない場合
                if ($order->get_data()['total'] != $params['Amount']) {
                    return true;
                }
            }

            return false;
        }

        /**
         * IPチェック
         * @return string
         */
        function checkIP()
        {
            if ($this->serverIP == '0.0.0.0' || $this->serverIP == '') {
                return '';
            }

            if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
                //check ip from share internet
                $ip = $_SERVER['HTTP_CLIENT_IP'];
            } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
                //to check ip is pass from proxy
                $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
            } else {
                $ip = $_SERVER['REMOTE_ADDR'];
            }

            // 不一致の場合
            if ($this->serverIP != $ip) {
                return $ip;
            }

            return '';
        }

    }
