<?php

/**
 * Exit if accessed directly
 */
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}
/*
   Plugin Name: アナザーレーン決済
   Description: WooCommerce対応のアナザーレーン決済プラグイン。
   Version: 1.2.0
   Author: アナザーレーン
   Author URI: https://www.alij.ne.jp/
   License: Under GPL2
*/

// 有効なプラグインと親クラスが読み込まれた後にロード 
// Include our Gateway Class and Register Payment Gateway with WooCommerce
add_action( 'plugins_loaded', 'woocommerce_alij_init', 0 );

function woocommerce_alij_init() {
    if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
        return;
   }

   // include our Gateway Class
   include_once( 'woocommerce-alij.php' );


   /**
    * Add this Gateway to WooCommerce
   **/
   add_filter( 'woocommerce_payment_gateways', 'woocommerce_add_alij_payment_gateway' );

   function woocommerce_add_alij_payment_gateway( $methods ) {
       $methods[] = 'WC_Alij_Payment';
       return $methods;
   }

}

// Add custom action links
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'alij_action_links');

function alij_action_links ( $links ) {
    $alij_links = array(
        '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout' ) . '">' . '設定' . '</a>',
   );
   return array_merge( $links, $alij_links );
}

// Declare Support For Cart+Checkout Blocks
add_action( 'before_woocommerce_init', function() {
    if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'cart_checkout_blocks', __FILE__, true );
    }
} );

// Blocks Support
add_action( 'woocommerce_blocks_loaded', function() {

	if( class_exists( 'Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType' ) ) {
		require_once( 'woocommerce-alij-blocks.php' );
		add_action(
			'woocommerce_blocks_payment_method_type_registration',
			function( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
				$payment_method_registry->register( new WC_Alij_Payment_Blocks );
		} );
	}

} );